; Test video character output and CPU speed
; Peter Dassow February 2021
; can be processed and linked with MASM 5.10 and MS LINK

.8086

CODE          SEGMENT  BYTE  PUBLIC
	      ASSUME   CS:CODE,DS:CODE

BENCH         PROC     NEAR
	      ORG      0100h

START1:       MOV      AX, CS             ; all data is located in CODE (CS)
	      MOV      DS, AX             ; so set DS to the same value
	      MOV      DX, Offset DispNotice
	      MOV      AH, 9
	      INT      21h
	      MOV      AH, 0              ; read a key -> AL
	      INT      16h
	      CMP      AL,1Bh             ; ESC pressed ?
	      JNE      START2
	      MOV      DX, Offset ExitPrg
	      MOV      AH,9
	      INT      21h                ; and displays exit message
	      JMP      EXITTHIS

START2:       MOV      AX,000Fh           ; get video mode
	      INT      10h
	      MOV      DI,BX              ; save BH with display page also
	      MOV      AX,0003h
	      INT      10h                ; clear screen MODE 3 (MODE CO80)
	      MOV      AX,0040h

	      MOV      ES,AX
	      MOV      AX,ES:[006Ch]      ; timer value (lower word)
	      MOV      DX,ES:[006eh]      ; (not for calculation, but must be saved)
	      MOV      [TicksLSW],AX      ; store it
	      MOV      [TicksMSW],DX      ; save it also for later restore
	      MOV      AX,0
	      MOV      ES:[006Ch],AX      ; reset both words of timer value
	      MOV      ES:[006Eh],AX

	      MOV      DL,0
	      MOV      CL,0B0h            ; 1st block graphics char
LOOP0:        MOV      AX,06E1h           ; 80 x 22 characters (= 1760 decimal) +1
LOOP1:        DEC      AX
	      CMP      AX,0               ; if counter reached 0 go to top again
	      JE       STOP1
	      PUSH     AX                 ; save counter
	      ;
	      PUSH     CX                 ; save also CX and DX because of
	      PUSH     DX                 ; the following INT 10h functions
	      ;
	      MOV      AH,09h             ; function 0E does not work in text mode
	      MOV      AL,CL              ; load pseudo graphics char
	      MOV      BX,DI              ; restore also current display page
	      MOV      BL,DL
	      MOV      CX,1               ; one char / one time
	      INT      10h
	      ;
	      MOV      AH,3               ; get cursor pos
	      ; BL still has the right display page
	      INT      10h
	      ;
	      INC      DL                 ; next char pos
	      CMP      DL,80              ; reached right screen border?
	      JB       CURS
	      MOV      DL,0h              ; back to most left position
	      INC      DH                 ; increment row
CURS:         MOV      AH,2               ; set cursor
	      INT      10h
	      ; now get rescued values back from stack
	      POP      DX
	      POP      CX                 ; restore CX value back (see above)
	      CMP      CL,0B2h            ; just to cycle the char from B0 to B2
	      JNE      LOOP2
	      MOV      CL,0AFh            ; reset the cycling char
LOOP2:        INC      CL
	      INC      DL                 ; increase color value
	      CMP      DL,10h             ; highest possible color is 0Fh
	      JB       LOOP3
	      MOV      DL,0               ; reset to color 0
LOOP3:        ; compare now tick values
	      MOV      AX, 0040h
	      MOV      ES,AX
	      MOV      AX, ES:[006Ch]     ; read lower word of timer value
	      CMP      AX,005Bh           ; 91 ticks means 5 seconds exactly
	      JGE      LOOP4              ; end of all
	      POP      AX
	      JMP      LOOP1
LOOP4:        POP      AX                 ; let the SP go back to normal
					  ; value of BX doesn't matter
	      MOV      AH,2
	      MOV      BX,DI              ; restore the saved display page
	      MOV      DX,1600h           ; cursor to y=20 x=0
	      INT      10h
	      JMP      STOP2

STOP1:        MOV      AX,[BigLoop]
	      CMP      AX,0FFFFh          ; this is for program safety only
	      JE       STOP2
	      INC      AX
	      MOV      [BigLoop],AX       ; increment it if not too much
	      PUSH     DX
	      MOV      AH,2
	      MOV      BX,DI              ; see above, saved display page
	      MOV      DX,0               ; Coords 0,0
	      INT      10h
	      POP      DX
	      JMP      LOOP0
	      ; Display difference between saved value and new value
STOP2:        MOV      AX, [BigLoop]
	      CALL     PRINT
	      ; now restore original timer values +91 ticks
	      MOV      AX,DS:[TicksMSW]
	      MOV      BX,0040h
	      MOV      ES,BX
	      MOV      ES:[006Eh],AX
	      ;
	      MOV      AX,DS:[TicksLSW]
	      ADD      AX,005Eh
	      ; MOV      ES,BX  ; not necessary again, still correct
	      MOV      ES:[006Ch],AX
	      ;
	      MOV      AH,9
	      MOV      DX,Offset TimeMSG
	      INT      21h

EXITTHIS:     MOV      AH, 4Ch
	      INT      21h                ; terminate program
BENCH         ENDP

print   proc
	;initialize count 
	mov cx,0 
	mov dx,0 
	; test for zero, if so, we can't divide
	cmp ax,0
	jne print1
	; if zero, just print a zero and leave
	mov dl,'0'
	mov ah,2
	int 21h
	jmp short exsub

print1:
	cmp ax,0
	je print2
	;initialize bx to 10 as the divisor
	mov bx,10

	; extract the last digit
	div bx

	;save dx for later use
	push dx

	;increment the count
	inc cx

	;set dx to 0
	xor dx,dx
	jmp short print1
print2:
	;check if count
	;is zero
	cmp cx,0
	je exsub

	;restore saved value
	pop dx

	;add 48 so that it
	;represents the ASCII
	;value of (decimal) digits
	add dx,48

	;display a digit
	mov ah,02h
	int 21h

	;decrease the count
	dec cx
	jmp short print2
exsub:
	ret ; leave the subroutine
PRINT ENDP 

TicksLSW      DW       0
TicksMSW      DW       0
BigLoop       DW       0
DispNotice    DB       "Benchmarking character CGA/EGA/VGA video output and CPU speed",13,10,13,10
	      DB       "Press any key to start or ESC to exit... $"
ExitPRG       DB       "<ESC>",13,10,'$'
TimeMSG       DB       " loops were reached from your PC,",13,10
	      DB       "measured with 91 ticks (5 secs).",13,10,13,10
	      DB       "Some values for comparison:",13,10,13,10
	      DB       "Intel Pentium 233 score = 1700 loops,",13,10
	      DB       "Intel 486DX @ 66MHz score = 360 loops.",13,10
	      DB       "Intel 386DX @ 33MHz score = 27 loops.",13,10
	      DB       "Intel 8088 @ 4.77MHz score = 4 loops.",13,10,'$'

CODE          ENDS
	      END      START1
